@@ -1,6 +1,8 @@
+use odbc_sys::SqlDataType;
+
 use crate::{
     buffers::Indicator,
-    handles::{Statement, StatementImpl},
+    handles::{State, Statement, StatementImpl},
     parameter::{VarBinarySliceMut, VarCharSliceMut},
     ColumnDescription, DataType, Error, Output,
 };
@@ -65,7 +67,13 @@ pub trait Cursor {
     /// driver in advance. Consider binding a buffer to the cursor first using
     /// [`Self::bind_buffer`].
     fn next_row(&mut self) -> Result<Option<CursorRow<'_, Self::Statement>>, Error> {
-        let row_available = unsafe { self.stmt().fetch()? };
+        let row_available = unsafe {
+            self.stmt()
+                .fetch()
+                .map(|res| res.into_result(self.stmt()))
+                .transpose()?
+                .is_some()
+        };
         let ret = if row_available {
             Some(CursorRow::new(unsafe { self.stmt() }))
         } else {
@@ -149,7 +157,9 @@ where
         col_or_param_num: u16,
         target: &mut impl Output,
     ) -> Result<(), Error> {
-        self.statement.get_data(col_or_param_num, target)
+        self.statement
+            .get_data(col_or_param_num, target)
+            .into_result(self.statement)
     }
 
     /// Retrieves arbitrary large character data from the row and stores it in the buffer. Column
@@ -340,7 +350,8 @@ where
     S: BorrowMut<StatementImpl<'o>>,
 {
     fn drop(&mut self) {
-        if let Err(e) = self.statement.borrow_mut().close_cursor() {
+        let stmt = self.statement.borrow_mut();
+        if let Err(e) = stmt.close_cursor().into_result(stmt) {
             // Avoid panicking, if we already have a panic. We don't want to mask the original
             // error.
             if !panicking() {
@@ -365,18 +376,19 @@ where
         column_number: u16,
         column_description: &mut ColumnDescription,
     ) -> Result<(), Error> {
-        self.statement
-            .borrow()
-            .describe_col(column_number, column_description)?;
-        Ok(())
+        let stmt = self.statement.borrow();
+        stmt.describe_col(column_number, column_description)
+            .into_result(stmt)
     }
 
     fn num_result_cols(&self) -> Result<i16, Error> {
-        self.statement.borrow().num_result_cols()
+        let stmt = self.statement.borrow();
+        stmt.num_result_cols().into_result(stmt)
     }
 
     fn is_unsigned_column(&self, column_number: u16) -> Result<bool, Error> {
-        self.statement.borrow().is_unsigned_column(column_number)
+        let stmt = self.statement.borrow();
+        stmt.is_unsigned_column(column_number).into_result(stmt)
     }
 
     fn bind_buffer<B>(mut self, mut row_set_buffer: B) -> Result<RowSetCursor<Self, B>, Error>
@@ -385,36 +397,119 @@ where
     {
         let stmt = self.statement.borrow_mut();
         unsafe {
-            stmt.set_row_bind_type(row_set_buffer.bind_type())?;
-            stmt.set_row_array_size(row_set_buffer.row_array_size())?;
-            stmt.set_num_rows_fetched(Some(row_set_buffer.mut_num_fetch_rows()))?;
+            stmt.set_row_bind_type(row_set_buffer.bind_type())
+                .into_result(stmt)?;
+            let size = row_set_buffer.row_array_size();
+            stmt.set_row_array_size(size)
+                .into_result(stmt)
+                // SAP anywhere has been seen to return with an "invalid attribute" error instead of
+                // a success with "option value changed" info. Let us map invalid attributes during
+                // setting row set array size to something more precise.
+                .map_err(|error| match error {
+                    Error::Diagnostics { record, .. }
+                        if record.state == State::INVALID_ATTRIBUTE_VALUE =>
+                    {
+                        Error::InvalidRowArraySize { record, size }
+                    }
+                    error => error,
+                })?;
+            stmt.set_num_rows_fetched(Some(row_set_buffer.mut_num_fetch_rows()))
+                .into_result(stmt)?;
             row_set_buffer.bind_to_cursor(&mut self)?;
         }
         Ok(RowSetCursor::new(row_set_buffer, self))
     }
 
     fn col_data_type(&self, column_number: u16) -> Result<DataType, Error> {
-        self.statement.borrow().col_data_type(column_number)
+        let stmt = self.statement.borrow();
+        let kind = stmt.col_concise_type(column_number).into_result(stmt)?;
+        let dt = match kind {
+            SqlDataType::UNKNOWN_TYPE => DataType::Unknown,
+            SqlDataType::EXT_VAR_BINARY => DataType::Varbinary {
+                length: self.col_octet_length(column_number)?.try_into().unwrap(),
+            },
+            SqlDataType::EXT_LONG_VAR_BINARY => DataType::LongVarbinary {
+                length: self.col_octet_length(column_number)?.try_into().unwrap(),
+            },
+            SqlDataType::EXT_BINARY => DataType::Binary {
+                length: self.col_octet_length(column_number)?.try_into().unwrap(),
+            },
+            SqlDataType::EXT_W_VARCHAR => DataType::WVarchar {
+                length: self.col_display_size(column_number)?.try_into().unwrap(),
+            },
+            SqlDataType::EXT_W_CHAR => DataType::WChar {
+                length: self.col_display_size(column_number)?.try_into().unwrap(),
+            },
+            SqlDataType::EXT_LONG_VARCHAR => DataType::LongVarchar {
+                length: self.col_display_size(column_number)?.try_into().unwrap(),
+            },
+            SqlDataType::CHAR => DataType::Char {
+                length: self.col_display_size(column_number)?.try_into().unwrap(),
+            },
+            SqlDataType::VARCHAR => DataType::Varchar {
+                length: self.col_display_size(column_number)?.try_into().unwrap(),
+            },
+            SqlDataType::NUMERIC => DataType::Numeric {
+                precision: self.col_precision(column_number)?.try_into().unwrap(),
+                scale: self.col_scale(column_number)?.try_into().unwrap(),
+            },
+            SqlDataType::DECIMAL => DataType::Decimal {
+                precision: self.col_precision(column_number)?.try_into().unwrap(),
+                scale: self.col_scale(column_number)?.try_into().unwrap(),
+            },
+            SqlDataType::INTEGER => DataType::Integer,
+            SqlDataType::SMALLINT => DataType::SmallInt,
+            SqlDataType::FLOAT => DataType::Float {
+                precision: self.col_precision(column_number)?.try_into().unwrap(),
+            },
+            SqlDataType::REAL => DataType::Real,
+            SqlDataType::DOUBLE => DataType::Double,
+            SqlDataType::DATE => DataType::Date,
+            SqlDataType::TIME => DataType::Time {
+                precision: self.col_precision(column_number)?.try_into().unwrap(),
+            },
+            SqlDataType::TIMESTAMP => DataType::Timestamp {
+                precision: self.col_precision(column_number)?.try_into().unwrap(),
+            },
+            SqlDataType::EXT_BIG_INT => DataType::BigInt,
+            SqlDataType::EXT_TINY_INT => DataType::TinyInt,
+            SqlDataType::EXT_BIT => DataType::Bit,
+            other => {
+                let mut column_description = ColumnDescription::default();
+                self.describe_col(column_number, &mut column_description)?;
+                DataType::Other {
+                    data_type: other,
+                    column_size: column_description.data_type.column_size(),
+                    decimal_digits: column_description.data_type.decimal_digits(),
+                }
+            }
+        };
+        Ok(dt)
     }
 
     fn col_octet_length(&self, column_number: u16) -> Result<isize, Error> {
-        self.statement.borrow().col_octet_length(column_number)
+        let stmt = self.statement.borrow();
+        stmt.col_octet_length(column_number).into_result(stmt)
     }
 
     fn col_display_size(&self, column_number: u16) -> Result<isize, Error> {
-        self.statement.borrow().col_display_size(column_number)
+        let stmt = self.statement.borrow();
+        stmt.col_display_size(column_number).into_result(stmt)
     }
 
     fn col_precision(&self, column_number: u16) -> Result<isize, Error> {
-        self.statement.borrow().col_precision(column_number)
+        let stmt = self.statement.borrow();
+        stmt.col_precision(column_number).into_result(stmt)
     }
 
     fn col_scale(&self, column_number: u16) -> Result<isize, Error> {
-        self.statement.borrow().col_scale(column_number)
+        let stmt = self.statement.borrow();
+        stmt.col_scale(column_number).into_result(stmt)
     }
 
     fn col_name(&self, column_number: u16, buf: &mut Vec<u16>) -> Result<(), Error> {
-        self.statement.borrow().col_name(column_number, buf)
+        let stmt = self.statement.borrow();
+        stmt.col_name(column_number, buf).into_result(stmt)
     }
 
     fn column_names(&self) -> Result<ColumnNamesIt<'_, Self>, Error> {
@@ -508,11 +603,13 @@ where
     /// `None` if the result set is empty and all row sets have been extracted. `Some` with a
     /// reference to the internal buffer otherwise.
     pub fn fetch(&mut self) -> Result<Option<&B>, Error> {
-        let has_data = unsafe { self.cursor.stmt().fetch()? };
-        if has_data {
-            Ok(Some(&self.buffer))
-        } else {
-            Ok(None)
+        unsafe {
+            if let Some(res) = self.cursor.stmt().fetch() {
+                res.into_result(self.cursor.stmt())?;
+                Ok(Some(&self.buffer))
+            } else {
+                Ok(None)
+            }
         }
     }
 }
@@ -526,7 +623,8 @@ where
             let stmt = self.cursor.stmt();
             if let Err(e) = stmt
                 .unbind_cols()
-                .and_then(|()| stmt.set_num_rows_fetched(None))
+                .into_result(stmt)
+                .and_then(|()| stmt.set_num_rows_fetched(None).into_result(stmt))
             {
                 // Avoid panicking, if we already have a panic. We don't want to mask the original
                 // error.
